//#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <stdint.h>
#include <vector>
#include <pcl/console/parse.h>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <pcl/io/ply_io.h>

#include <pcl/io/obj_io.h>
#include <pcl/io/vtk_lib_io.h>
#include <pcl/PolygonMesh.h>

using namespace std;

int
main(int argc, char** argv)
{

	pcl::PointCloud<pcl::PointXYZRGB> cloud;
	pcl::PointXYZRGB p;

	int r, g, b;
	double ref;
	ifstream ins;
	bool ispts = false;
	int skipline = 9;

	int interval = 1;
	pcl::console::parse_argument(argc, argv, "-i", interval);

	double maxdepth = 0;
	pcl::console::parse_argument(argc, argv, "-d", maxdepth);

	bool ply = false;
	ply = pcl::console::find_switch(argc, argv, "--ply");

	bool display_help = false;
	display_help = pcl::console::find_switch(argc, argv, "--help");

	if (display_help || argc <= 1)
	{
		std::cout << "Usage: ptxrgb2pcd.exe [OPTION]... [ptx]" << std::endl;
		std::cout << "  -i          interval" << std::endl;
		std::cout << "  -d          max depth" << std::endl;
		std::cout << "     --ply    save as ply file" << std::endl;
		std::cout << "     --help   display this help and exit" << std::endl;

		return (0);
	}

	std::cout << "Interval  : " << interval << std::endl;
	std::cout << "Maximum depth : " << maxdepth << std::endl;
	std::cout << "Save as ply file : " << ply << std::endl;

	std::vector<int> pcd_indices;
	pcd_indices = pcl::console::parse_file_extension_argument(argc, argv, ".ptx");

	if (pcd_indices.size() < 1) {
		pcd_indices = pcl::console::parse_file_extension_argument(argc, argv, ".pts");
		if (pcd_indices.size() >= 1) {
		skipline = 8;
		ispts = true;
		}
	}

	if (pcd_indices.size() < 1) {
		std::cout << "ptx file must be specified" << std::endl;
		return (0);
	}

	// ********************* t@C̓ǂݍ *********************
	std::string infile = argv[pcd_indices[0]];
	std::string outfile = argv[pcd_indices[0]];

	if (ply)
		outfile.replace(outfile.length() - 3, 3, "ply");
	else
		outfile.replace(outfile.length() - 3, 3, "pcd");


	ins.open(infile.c_str());

	if(!ins)
	{
		// Opening a file encounters a problem
		cerr << "Error: file could not be opened!" << endl;
		exit(1);
	}
	else
	{
		int n = 0;
		string buf;
		while (ins && getline(ins, buf)) if (n++ >= skipline) break;

		n = 0;
		while(!ins.eof())
		{
			ins >> p.x;
			ins >> p.y;
			ins >> p.z;
			ins >> ref;
			if (ispts){
				r = g = b = 0;
			}
			else{
				ins >> r;
				ins >> g;
				ins >> b;
			}
			p.r = (uint8_t)r;
			p.g = (uint8_t)g;
			p.b = (uint8_t)b;

			if (n++%interval) continue;
			if (maxdepth > 0.0 && sqrt(p.x*p.x + p.y*p.y + p.z*p.z) > maxdepth) continue;
			if (p.x != 0.0 || p.y != 0.0|| p.z != 0.0) cloud.points.push_back(p);
		}

		cloud.width = cloud.points.size();
		cloud.height = 1;
		cloud.points.resize(cloud.width*cloud.height);

		ins.close();

		if (ply){
			bool binary = true;
			bool use_camera = false;
			pcl::PLYWriter w;
			w.write(outfile, cloud, binary, use_camera);
		}
		else {
			pcl::io::savePCDFileASCII(outfile, cloud);
		}
	}

	return 0;
} 
